.ui-sortable-placeholder {
    height: 0;
    overflow: hidden;
}

.clearfix:after {
   content: " "; /* Older browser do not support empty content */
   visibility: hidden;
   display: block;
   height: 0;
   clear: both;
}

.prevent-scroll {
    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
    outline: none;
    cursor: default;
}

.fc-item {
    clear: both;
}
.fc-item .simple,
.fc-item .advanced {
    background-color: white;
    width: 100%;
    padding: 4px 0;
}
.fc-item .advanced {
    display: none;
}
.fc-item .simple label,
.fc-item .advanced label {
    display: block;
    padding: 4px 10px;
}
.fc-item label > h4 {
    display: block;
    float: left;
    clear: left;
    width: 10%;
    font-weight: normal;
}
.fc-item label > .field-wrapper {
    float: left;
    display: block;
    width: 89%;
    margin: 0 0 0 1%; 
    padding: 0;
}
.fc-item label input[type=text] {
    width: 50%;
}
.fc-item label textarea {
    width: 50%;
    height: 50px;
    margin: 0;
    padding: 0;
}
/*.fc-item label span.cke_skin_kama { TODO: remove me!
    border: none;
    padding: 0;
    margin: 0;
}*/
.fc-item.fc-deleted .simple,
.fc-item.fc-deleted .advanced {
    position: relative;
    z-index: 0;
}
.fc-item.fc-deleted .simple::before,
.fc-item.fc-deleted .advanced::before {
    content: '';
    position: absolute;
    z-index: 1;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    background-color: #fff;
    opacity: 0.7;
}
.fc-item .header {
    background-color: #eee;
    position: relative;
    overflow: hidden;
    height: 30px;
    border-top: 1px solid #bbb;
    border-bottom: 1px solid #ddd;
}
.fc-item .header input.fc-ordering {
    position: absolute;
    z-index: 1;
    top: -16px;
    left: 20px;
    background: none;
    border: none;
    font-weight: bold;
    font-size: 44px;
    color: #bbb;
}
.fc-item .header h3 {
    position: absolute;
    z-index: 2;
    top: 0;
    left: 0;
    display: block;
    margin: 0;
    padding: 7px 0 7px 100px;
    width: 300px;
    color: #333;
    font-weight: bold;
    font-size: 13px;
}
.fc-item .header menu {
    position: absolute;
    top: 7px;
    right: 10px;
    display: block;
    margin: 0;
    font-weight: 11px;
    /* opacity */
    -ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=25)";
    filter: alpha(opacity=25);
    opacity: 0.25;
}
.fc-item:hover .header menu {
    -ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=100)";
    filter: alpha(opacity=100);
    opacity: 1.0;
}
.fc-item .header menu a {
    display: inline-block;
    margin-left: 15px;
}
.fc-item a.fc-delete {
    color: #c00;
}

.fc-add-item p {
    margin-bottom: 3px;
    font-weight: bold;
}
ul.fc-item-types li {
    padding: 0;
}
ul.fc-item-types a {
    display: block;
    font-size: 12px;
    font-weight: bold;
    padding: 2px 0;
}
ul.fc-item-types a span {
    font-weight: normal;
    color: #000;
}
